@@ -61,7 +61,7 @@ def full():
     tMarks = np.append(tMarks, time.time())
 
     # Loop over remaining test profiles (2 onwards)
-    for i in range(2, nTests+1):
+    for i in range(13, nTests+1):
         testPlanetN = importlib.import_module(f'{testBase}{i}').Planet
         log.info(f'Test case body: {testBase}{i}')
         TestPlanets = np.append(TestPlanets, PlanetProfile(testPlanetN, Params)[0])
@@ -290,21 +290,22 @@ def simple():
     Params.RUN_ALL_PROFILES = False
     Params.COMPARE = False
     Params.DO_INDUCTOGRAM = False
-    Params.DO_EXPLOREOGRAM = True
+    Params.DO_EXPLOREOGRAM = False
     Params.DO_PARALLEL = False
     Params.SKIP_INDUCTION = False
 
-    iTest = 3
-    bodyname = f'{testMod}{iTest}'
+    iTests = [13]
     tStart = time.time()
-    testPlanet = importlib.import_module(bodyname).Planet
-    log.info(f'Test case body: {bodyname}')
-    if Params.DO_INDUCTOGRAM:
-        TestInductOgram(iTest, Params, CALC_NEW=Params.CALC_NEW_INDUCT)
-    elif Params.DO_EXPLOREOGRAM:
-        TestExploreOgram(iTest, Params, CALC_NEW=Params.CALC_NEW_INDUCT)
-    else:
-        _ = PlanetProfile(testPlanet, Params)
+    for iTest in iTests:
+        bodyname = f'{testMod}{iTest}'
+        testPlanet = importlib.import_module(bodyname).Planet
+        log.info(f'Test case body: {bodyname}')
+        if Params.DO_INDUCTOGRAM:
+            TestInductOgram(iTest, Params, CALC_NEW=Params.CALC_NEW_INDUCT)
+        elif Params.DO_EXPLOREOGRAM:
+            TestExploreOgram(iTest, Params, CALC_NEW=Params.CALC_NEW_INDUCT)
+        else:
+            _ = PlanetProfile(testPlanet, Params)
     tEnd = time.time()
     log.info('Simple test complete!')
 
