@@ -6,8 +6,8 @@ from scipy.optimize import root_scalar as GetZero
 from scipy.io import loadmat
 from seafreeze import seafreeze as SeaFreeze
 from seafreeze import whichphase as WhichPhase
-from PlanetProfile.Thermodynamics.Clathrates.ClathrateProps import ClathProps, ClathStableSloan1998, TclathDissocLower_K, \
-    TclathDissocUpper_K, ClathSeismic
+from PlanetProfile.Thermodynamics.Clathrates.ClathrateProps import ClathProps, ClathStableSloan1998, \
+    ClathStableNagashima2017, ClathSeismic
 from PlanetProfile.Thermodynamics.InnerEOS import GetphiFunc, GetphiCalc, ResetNearestExtrap, ReturnZeros, EOSwrapper
 from PlanetProfile.Thermodynamics.MgSO4.MgSO4Props import MgSO4Props, MgSO4PhaseMargules, MgSO4PhaseLookup, \
     MgSO4Seismic, MgSO4Conduct
@@ -18,9 +18,9 @@ from PlanetProfile.Utilities.defineStructs import Constants, EOSlist
 log = logging.getLogger('PlanetProfile')
 
 def GetOceanEOS(compstr, wOcean_ppt, P_MPa, T_K, elecType, rhoType=None, scalingType=None, phaseType=None,
-                EXTRAP=False, FORCE_NEW=False):
+                EXTRAP=False, FORCE_NEW=False, MELT=False):
     oceanEOS = OceanEOSStruct(compstr, wOcean_ppt, P_MPa, T_K, elecType, rhoType=rhoType, scalingType=scalingType,
-                              phaseType=phaseType, EXTRAP=EXTRAP, FORCE_NEW=FORCE_NEW)
+                              phaseType=phaseType, EXTRAP=EXTRAP, FORCE_NEW=FORCE_NEW, MELT=MELT)
     if oceanEOS.ALREADY_LOADED:
         log.debug(f'{wOcean_ppt} ppt {compstr} EOS already loaded. Reusing existing EOS.')
         oceanEOS = EOSlist.loaded[oceanEOS.EOSlabel]
@@ -36,7 +36,7 @@ def GetOceanEOS(compstr, wOcean_ppt, P_MPa, T_K, elecType, rhoType=None, scaling
 
 class OceanEOSStruct:
     def __init__(self, compstr, wOcean_ppt, P_MPa, T_K, elecType, rhoType=None, scalingType=None,
-                 phaseType=None, EXTRAP=False, FORCE_NEW=False):
+                 phaseType=None, EXTRAP=False, FORCE_NEW=False, MELT=False):
         if elecType is None:
             self.elecType = 'Vance2018'
         else:
@@ -53,8 +53,15 @@ class OceanEOSStruct:
             self.PHASE_LOOKUP = True
         else:
             self.PHASE_LOOKUP = False
+        # Add ID for melting curve EOS
+        if MELT:
+            meltStr = f'melt{np.max(P_MPa)}'
+            meltPrint = 'melting curve '
+        else:
+            meltStr = ''
+            meltPrint = ''
 
-        self.EOSlabel = f'{compstr}{wOcean_ppt}{elecType}{rhoType}{scalingType}{phaseType}{EXTRAP}'
+        self.EOSlabel = f'{meltStr}{compstr}{wOcean_ppt}{elecType}{rhoType}{scalingType}{phaseType}{EXTRAP}'
         self.ALREADY_LOADED, self.rangeLabel, P_MPa, T_K, self.deltaP, self.deltaT \
             = CheckIfEOSLoaded(self.EOSlabel, P_MPa, T_K, FORCE_NEW=FORCE_NEW)
 
@@ -72,7 +79,7 @@ class OceanEOSStruct:
                 wStr = '0.0'
             else:
                 wStr = f'{wOcean_ppt:.1f}'
-            log.debug(f'Loading EOS for {wStr} ppt {compstr} with ' +
+            log.debug(f'Loading {meltPrint}EOS for {wStr} ppt {compstr} with ' +
                       f'P_MPa = [{self.Pmin:.1f}, {self.Pmax:.1f}, {self.deltaP:.2f}], ' +
                       f'T_K = [{self.Tmin:.1f}, {self.Tmax:.1f}, {self.deltaT:.2f}], ' +
                       f'for [min, max, step] with EXTRAP = {self.EXTRAP}.')
@@ -200,9 +207,11 @@ class OceanEOSStruct:
         return self.ufn_sigma_Sm(P_MPa, T_K, grid=grid)
 
 
-def GetIceEOS(P_MPa, T_K, phaseStr, porosType=None, phiTop_frac=0, Pclosure_MPa=0, phiMin_frac=0, EXTRAP=False):
+def GetIceEOS(P_MPa, T_K, phaseStr, porosType=None, phiTop_frac=0, Pclosure_MPa=0, phiMin_frac=0, EXTRAP=False,
+              ClathDissoc=None):
     iceEOS = IceEOSStruct(P_MPa, T_K, phaseStr, porosType=porosType, phiTop_frac=phiTop_frac,
-                          Pclosure_MPa=Pclosure_MPa, phiMin_frac=phiMin_frac, EXTRAP=EXTRAP)
+                          Pclosure_MPa=Pclosure_MPa, phiMin_frac=phiMin_frac, EXTRAP=EXTRAP,
+                          ClathDissoc=ClathDissoc)
     if iceEOS.ALREADY_LOADED:
         log.debug(f'Ice {phaseStr} EOS already loaded. Reusing existing EOS.')
         iceEOS = EOSlist.loaded[iceEOS.EOSlabel]
@@ -217,7 +226,8 @@ def GetIceEOS(P_MPa, T_K, phaseStr, porosType=None, phiTop_frac=0, Pclosure_MPa=
     return iceEOSwrapper
 
 class IceEOSStruct:
-    def __init__(self, P_MPa, T_K, phaseStr, porosType=None, phiTop_frac=0, Pclosure_MPa=0, phiMin_frac=0, EXTRAP=False):
+    def __init__(self, P_MPa, T_K, phaseStr, porosType=None, phiTop_frac=0, Pclosure_MPa=0, phiMin_frac=0, EXTRAP=False,
+                 ClathDissoc=None):
         self.EOSlabel = f'{phaseStr}{porosType}{phiTop_frac}{Pclosure_MPa}{phiMin_frac}{EXTRAP}'
         self.ALREADY_LOADED, self.rangeLabel, P_MPa, T_K, self.deltaP, self.deltaT \
             = CheckIfEOSLoaded(self.EOSlabel, P_MPa, T_K)
@@ -254,7 +264,10 @@ class IceEOSStruct:
                 # Special functions for clathrate properties
                 rho_kgm3, Cp_JkgK, alpha_pK, kTherm_WmK \
                     = ClathProps(P_MPa, T_K)
-                self.phase = ClathStableSloan1998(P_MPa, T_K)
+                if ClathDissoc is not None and ClathDissoc.NAGASHIMA:
+                    self.phase = ClathStableNagashima2017(P_MPa, T_K)
+                else:
+                    self.phase = ClathStableSloan1998(P_MPa, T_K)
 
                 Plin_MPa = np.array([P for P in P_MPa for _ in T_K])
                 Tlin_K = np.array([T for _ in P_MPa for T in T_K])
@@ -271,7 +284,7 @@ class IceEOSStruct:
                 rho_kgm3 = iceOut.rho
                 Cp_JkgK = iceOut.Cp
                 alpha_pK = iceOut.alpha
-                kTherm_WmK = np.array([kThermIsobaricAnderssonIbari2005(T_K, PhaseInv(phaseStr)) for _ in P_MPa])
+                kTherm_WmK = np.array([kThermIsothermalAnderssonIbari2005(P_MPa, PhaseInv(phaseStr)) for _ in T_K]).T
                 self.ufn_Seismic = IceSeismic(phaseStr, self.EXTRAP)
                 self.fn_phase = returnVal(self.phaseID)
 
@@ -756,36 +769,3 @@ def kThermHobbs1974(T_K):
 
     return kTherm_WmK
 
-
-def GetPbClath(Tb_K, ALLOW_BROKEN_MODELS=False, DO_EXPLOREOGRAM=False):
-    """ Calculate the pressure consistent with Tb_K when clathrates are assumed
-        to be in contact with the ocean, i.e. for Bulk.clathType = 'bottom' or 'whole'.
-
-        Args:
-            Tb_K (float): Clathrate layer bottom temperature in K
-        Returns:
-            PbClath_MPa (float): Bottom temperature consistent with dissociation curve
-                pressure at Tb_K
-    """
-    if Tb_K < 273:
-        TbZero_K = lambda P_MPa: Tb_K - TclathDissocLower_K(P_MPa)
-        Pends_MPa = [0.0, 2.567]
-    else:
-        TbZero_K = lambda P_MPa: Tb_K - TclathDissocUpper_K(P_MPa)
-        Pends_MPa = [2.567, Constants.PmaxLiquid_MPa]
-
-    try:
-        PbClath_MPa = GetZero(TbZero_K, bracket=Pends_MPa).root
-    except ValueError:
-        msg = f'No Pb was found for clathrates with Tb_K = {Tb_K}.'
-        if ALLOW_BROKEN_MODELS:
-            if DO_EXPLOREOGRAM:
-                log.info(msg)
-            else:
-                log.error(msg + 'ALLOW_BROKEN_MODELS is True, so calculations will proceed, with many values set to nan.')
-            PbClath_MPa = np.nan
-        else:
-            raise ValueError(msg)
-
-    return PbClath_MPa
-
